/*ht1632.c*/

#include "ht1632.h"

extern struct HT1632* ht1632;

//initialize panel attributes
struct HT1632* ht1632Init(uint8_t data, uint8_t wr,uint8_t cs,int8_t rd){



ht1632->_data=data;
ht1632->_cs=cs;
ht1632->_wr=wr;
ht1632->_rd=rd;

//intialize panel leds to 0
for(uint8_t i=0;i<48;i++){
	ht1632->ledmatrix[i]=0;

return ht1632;

}//end for

}//end ht1632

//initialize panel
void begin(uint8_t type, struct HT1632* ht1632){
 //set _cs pin as output
 DDRB |=(1<<ht1632->_cs);
 //make cs high
 PORTB |=(1<<ht1632->_cs);
 //set _wr pin to output
 DDRB |=(1<<ht1632->_wr);
 //make wr high
 PORTB |=(1<<ht1632->_wr);
 //make _DATA as output
 DDRB |=(1<<ht1632->_data);

 if(ht1632->_rd >= 0){
 	//set _RD to output
	DDRB |=(1<<ht1632->_rd);
	//make _RD high
	PORTB |=(1<<ht1632->_rd);
 }//end if

  sendcommand(HT1632_SYS_EN, ht1632);
  sendcommand(HT1632_LED_ON, ht1632);
  sendcommand(HT1632_BLINK_OFF, ht1632);
  sendcommand(HT1632_MASTER_MODE, ht1632);
  sendcommand(HT1632_INT_RC, ht1632);
  sendcommand(type, ht1632);
  sendcommand(HT1632_PWM_CONTROL | 0xF, ht1632);
  
  ht1632->WIDTH = 24;
  ht1632->HEIGHT = 16;


}//end begin


void clrPixel(uint16_t i,struct HT1632* ht1632) {
  ht1632->ledmatrix[i/8] &= ~_BV(i%8); 
}//end clear pixel

void setPixel(uint16_t i, struct HT1632* ht1632) {
  ht1632->ledmatrix[i/8] |= _BV(i%8); 
}//end setPixel

void blink(bool state, struct HT1632* ht1632) {
  if (state) 
    sendcommand(HT1632_BLINK_ON, ht1632);
  else
    sendcommand(HT1632_BLINK_OFF, ht1632);
}//end blink


void setBrightness(uint8_t pwm, struct HT1632* ht1632) {
  if (pwm > 15) pwm = 15;
  sendcommand(HT1632_PWM_CONTROL | pwm, ht1632);
}//end setBrightness


void clearScreen(struct HT1632* ht1632) {
  for (uint8_t i=0; i<(ht1632->WIDTH*ht1632->HEIGHT/8); i++) {
    ht1632->ledmatrix[i] = 0;
  }//end for
  writeScreen(ht1632);
}//end clear Screen


void fillScreen(struct HT1632* ht1632) {
  for (uint8_t i=0; i<(ht1632->WIDTH*ht1632->HEIGHT/8); i++) {
    ht1632->ledmatrix[i] = 0xFF;
  }//end for
  writeScreen(ht1632);
}//end fill screen


void writeScreen(struct HT1632* ht1632) {
	//make cs low
 	PORTB &= ~(1<<ht1632->_cs);
 
  writedata(HT1632_WRITE, 3, ht1632);
  // send with address 0
  writedata(0, 7, ht1632);

  for (uint16_t i=0; i<(ht1632->WIDTH*ht1632->HEIGHT/8); i+=2) {
    uint16_t d = ht1632->ledmatrix[i];
    d <<= 8;
    d |= ht1632->ledmatrix[i+1];

    writedata(d, 16, ht1632);
  }//end for

	//make cs high
 	PORTB |= (1<<ht1632->_cs);
}//end write scrren



void sendcommand(uint8_t cmd, struct HT1632* ht1632){
	uint16_t data=0;
	data = HT1632_COMMAND;
	data <<= 8;
	data |= cmd;
	data <<=1;

	//latch sequence
	//make cs low
 	PORTB &= ~(1<<ht1632->_cs);
	writedata(data,12, ht1632);
	//make cs high
    PORTB |= (1<<ht1632->_cs);

}//end send command


void writedata(uint16_t d, uint8_t bits, struct HT1632* ht1632){
	//set _data pin as output
    DDRB |= (1<<ht1632->_data);

	for(uint8_t i=bits;i>0;i--){
		//make wr low
 		PORTB &= ~(1<<ht1632->_wr);
		
		if(d & (1<<(i-1))){
			//make _data high
			 PORTB |= (1<<ht1632->_data);
		}//end if
		
		else{
			//make _data low
 			PORTB &= ~(1<<ht1632->_data);
		}//end else		
	
		//make _wr high
		PORTB |= (1<<ht1632->_wr);

	}//end for

	//set _data pin as input
    DDRB &= ~(1<<ht1632->_data);

}//end write data


void writeRAM(uint8_t addr, uint8_t data, struct HT1632* ht1632) {
  //Serial.print("Writing 0x"); Serial.print(data&0xF, HEX);
  //Serial.print(" to 0x"); Serial.println(addr & 0x7F, HEX);

  uint16_t d = HT1632_WRITE;
  d <<= 7;
  d |= addr & 0x7F;
  d <<= 4;
  d |= data & 0xF;
 
  //set _cs pin low
  PORTB &= ~(1<<ht1632->_cs);

  writedata(d, 14, ht1632);

  //set _cs pin high
  PORTB |= (1<<ht1632->_cs);

}//end writeRAM






