/*ht1632.h*/

#ifndef ht1632_h
#define ht1632_h

#include <avr\io.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include "glcdfont.c"

/*Definition of a linkedList structure*/

struct HT1632 {
   //"Object" variables(From ht1632)
   uint8_t WIDTH;
   uint8_t HEIGHT;
   uint8_t _data;
   uint8_t _cs;
   uint8_t _wr;
   int8_t _rd;
   uint8_t ledmatrix[48];     // 16 * 24 / 8
};

struct HT1632* ht1632;

//definitions
#define HT1632_READ  0x6
#define HT1632_WRITE 0x5
#define HT1632_COMMAND 0x4

#define HT1632_SYS_DIS 0x00
#define HT1632_SYS_EN 0x01
#define HT1632_LED_OFF 0x02
#define HT1632_LED_ON 0x03
#define HT1632_BLINK_OFF 0x08
#define HT1632_BLINK_ON 0x09
#define HT1632_SLAVE_MODE 0x10
#define HT1632_MASTER_MODE 0x14
#define HT1632_INT_RC 0x18
#define HT1632_EXT_CLK 0x1C
#define HT1632_PWM_CONTROL 0xA0

#define HT1632_COMMON_8NMOS  0x20
#define HT1632_COMMON_16NMOS  0x24
#define HT1632_COMMON_8PMOS  0x28
#define HT1632_COMMON_16PMOS  0x2C


/**********************************************
*HT1632 FUNCTIONS
**********************************************/

//initialize panel attributes
//must pass in -1 if not declared in examples
struct HT1632* ht1632Init(uint8_t data, uint8_t wr,uint8_t cs,int8_t rd);

//initialize panel
void begin(uint8_t type, struct HT1632* ht1632);

void clrPixel(uint16_t i, struct HT1632* ht1632);
void setPixel(uint16_t i, struct HT1632* ht1632);
void blink(bool state, struct HT1632* ht1632);
void setBrightness(uint8_t pwm, struct HT1632* ht1632);
void clearScreen(struct HT1632* ht1632);
void fillScreen(struct HT1632* ht1632);
void writeScreen(struct HT1632* ht1632);
void dumpScreen(struct HT1632* ht1632);
void sendcommand(uint8_t cmd,struct HT1632* ht1632);
void writedata(uint16_t d, uint8_t bits, struct HT1632* ht1632);
void writeRAM(uint8_t addr, uint8_t data, struct HT1632* ht1632);



#endif
